<?php namespace EwCacheInvalidator;

use OcList;
use EveryBoard_WidgetPrefab;

/**
 * Class PageQueries
 * @package EwCacheInvalidator
 */
class PageQueries {

    /**
     * PageQueries constructor.
     */
    public function __construct() {

    }

    /**
     * Get all queries.
     *
     * @return array
     */
    public function getAllQueries() {
        return $this->getQueries();
    }

    /**
     * Get queries for given page.
     *
     * @param $page_id
     * @return array
     */
    public function getPageQueries($page_id) {
        return $this->getQueries($page_id);
    }

    /**
     * Get queries optional page id.
     *
     * @param int $page_id
     * @return array
     */
    private function getQueries($page_id = 0) {

        $pages = [];
        if($page_id === 0) {
            $pages = $this->getAllPages();
        }
        else {
            $pages[] = $this->getPage($page_id);
        }

        $query_arr = [];
        foreach($pages as $page) {

            $board_id = get_post_meta($page->ID, 'everyboard_id', true);

            global $post;
            $post = $page;

            if($board_id !== "") {

                $board_json = get_post_meta($board_id, 'everyboard_json', true);
                $board_data = json_decode($board_json);

                if(isset($board_data->rows)) {

                    foreach($board_data->rows as $row) {

                        foreach($row->cols as $col) {

                            if(isset($col->content)) {
                                foreach($col->content as $item) {

                                    if ( isset( $item->type ) ) {
                                        switch ($item->type) {
                                            case 'widget':

                                                $widget_query = $this->getWidgetQuery($item);
                                                if($widget_query !== false && $widget_query !== '' && isset($widget_query)) {

                                                    if(!array_key_exists($widget_query, $query_arr)) {
                                                        $query_arr[$widget_query] = [];
                                                    }

                                                    $query_arr[] = [$widget_query, $page->ID, 'widget'];
                                                }
                                                break;
                                            case 'linked_widget';
                                                // extract data from this as well?
                                                break;
                                            case "oclist_widget":
                                                $list_queries = $this->getOcListQuery($item);
                                                if(count($list_queries) > 0) {

                                                    foreach($list_queries as $list_query) {
                                                        if($list_query !== '') {
                                                            if(!array_key_exists($list_query, $query_arr)) {
                                                                $query_arr[$list_query] = [];
                                                            }

                                                            $query_arr[] = [$list_query, $page->ID, 'oclist'];
                                                        }
                                                    }
                                                }

                                                break;
                                            case "contentcontainer_widget":

                                                // filter run to extract query if possible?
                                                $cc_query = $this->getCcQuery($item);
                                                if($cc_query !== false && $cc_query !== '') {
                                                    if(!array_key_exists($cc_query, $query_arr)) {
                                                        $query_arr[$cc_query] = [];
                                                    }

                                                    $query_arr[] = [$cc_query, $page->ID, 'contentcontainer'];
                                                }
                                                break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        $query_arr = array_unique($query_arr, SORT_REGULAR);
        return $query_arr;
    }

    /**
     * Retrieve all wordpress pages.
     *
     * @return array|false
     */
    private function getAllPages() {

        return get_pages([
            'posts_per_page' => -1
        ]);
    }

    /**
     * Get page by given id.
     */
    private function getPage($page_id) {
        return get_post($page_id);
    }

    /**
     * Get Content Container query.
     *
     * @param $item
     * @return bool|mixed
     */
    private function getCcQuery($item) {

        if ( isset( $item ) ) {
            $cc_query = apply_filters( 'ew_content_container_cache_query', [ ] );
            if(count($cc_query) > 0) {
                return $cc_query;
            }
        }

        return false;
    }

    /**
     * Get OC List query.
     *
     * @param $item
     * @return array
     */
    private function getOcListQuery($item) {

        $return_arr = [];
        if(isset($item->settings->list) && $item->settings->list !== "0") {

            $query = OcList::get_approved_list_articles_query($item->settings->list);
            $parsed_query = parse_url($query);

            if($parsed_query !== false && isset($parsed_query['query'])) {

                $query = $parsed_query['query'];
                parse_str($query, $query_str);

                $list_articles = OcList::get_approved_list_articles($item->settings->list);
                $list_articles_uuids_query = '';
                foreach($list_articles as $article) {

                    if(isset($article->uuid[0])) {
                        $list_articles_uuids_query .= 'uuid:' . $article->uuid[0] . ' OR ';
                    }
                }

                if(strlen($list_articles_uuids_query) > 0) {
                    $return_arr[] = substr($list_articles_uuids_query, 0, -4);
                }

                if(isset($query_str['q']) && $query_str['q'] !== "0") {
                    $return_arr[] = $query_str['q'];
                }
            }
        }

        return $return_arr;
    }

    /**
     * Get widget query.
     *
     * @param $item
     * @return bool
     */
    private function getWidgetQuery($item) {

        if ( isset( $item->widget_id ) ) {
            $widget_board_instance = EveryBoard_WidgetPrefab::get_instance()->get_widget_instance($item->widget_id);
            return self::getQueryFromWidgetInstance($item->widget_id, $widget_board_instance['settings']);
        }

        return false;
    }

    /**
     * Get widget instance.
     *
     * @param $id_base
     * @param array $settings
     * @param null $sidebar_id
     * @return bool
     */
    public static function getWidgetInstance( $id_base, array $settings = array(), $sidebar_id = null ) {

        global $wp_registered_widgets;
        static $multi_number = 0;
        $multi_number++;

        $sidebars = wp_get_sidebars_widgets();

        if ( isset( $sidebar_id ) ) {

            $sidebar = ( isset( $sidebars[ $sidebar_id ] ) ) ? $sidebars[ $sidebar_id ] : array();

        } else {

            $sidebar_id = key( $sidebars );
            $sidebar = array_shift( $sidebars );
        }

        $sidebar[] = $id_base . '-' . $multi_number;

        $_POST['sidebar'] = $sidebar_id;
        $_POST[ "widget-{$id_base}" ] = array( $multi_number => $settings );
        $_POST['widget-id'] = $sidebar;

        if(!isset($wp_registered_widgets[ $id_base ]) || !is_callable($wp_registered_widgets[ $id_base ]['callback'])) {
            return false;
        }

        $control = $wp_registered_widgets[ $id_base ];

        return $control['callback'][0];
    }

    /**
     * Get query from a widget instance.
     *
     * @param $widget_id
     * @param $widget_settings
     * @return mixed
     */
    public static function getQueryFromWidgetInstance($widget_id, $widget_settings) {

        if(isset($widget_settings)) {

            $widget_instance = self::getWidgetInstance($widget_id, $widget_settings, 'everyboard_widgets');
            if(method_exists($widget_instance, 'get_oc_query')) {
                $query = $widget_instance->get_oc_query($widget_settings);

                $parsed_query = parse_url($query);
                if($parsed_query !== false && isset($parsed_query['query'])) {

                    $query = $parsed_query['query'];
                    parse_str($query, $query_str);

                    if(isset($query_str['q']) && $query_str['q'] !== '') {
                        return $query_str['q'];
                    }
                }
            }
        }
    }
}